@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.39';
+our $VERSION = '0.40';
 
 our $DEFAULT_UA = "Perl + " . __PACKAGE__ . "/$VERSION";
 our $DEFAULT_MAXREDIR = 3;
@@ -218,6 +218,8 @@ cookies in requests and store them from responses.
 
 Optional. If false, disables HTTP/1.1-style request pipelining.
 
+=head2 family => INT
+
 =head2 local_host => STRING
 
 =head2 local_port => INT
@@ -227,8 +229,9 @@ Optional. If false, disables HTTP/1.1-style request pipelining.
 =head2 local_addr => HASH or ARRAY
 
 Optional. Parameters to pass on to the C<connect> method used to connect
-sockets to HTTP servers. Sets the local socket address to C<bind()> to. For
-more detail, see the documentation in L<IO::Async::Connector>.
+sockets to HTTP servers. Sets the socket family and local socket address to
+C<bind()> to. For more detail, see the documentation in
+L<IO::Async::Connector>.
 
 =head2 fail_on_error => BOOL
 
@@ -294,7 +297,7 @@ sub configure
 
    foreach (qw( user_agent max_redirects max_in_flight max_connections_per_host
       timeout stall_timeout proxy_host proxy_port cookie_jar pipeline
-      local_host local_port local_addrs local_addr fail_on_error
+      family local_host local_port local_addrs local_addr fail_on_error
       read_len write_len decode_content require_SSL ))
    {
       $self->{$_} = delete $params{$_} if exists $params{$_};
@@ -357,7 +360,9 @@ sub connect_connection
    my $f = $conn->connect(
       host     => $host,
       service  => $port,
-      ( map { defined $self->{$_} ? ( $_ => $self->{$_} ) : () } qw( local_host local_port local_addrs local_addr ) ),
+      family   => ( $args{family} || $self->{family} || 0 ),
+      ( map { defined $self->{$_} ? ( $_ => $self->{$_} ) : () }
+         qw( local_host local_port local_addrs local_addr ) ),
 
       %args,
    )->on_done( sub {
@@ -477,6 +482,11 @@ Hostname of the server to connect to
 Optional. Port number or service of the server to connect to. If not defined,
 will default to C<http> or C<https> depending on whether SSL is being used.
 
+=item family => INT
+
+Optional. Restricts the socket family for connecting. If not defined, will
+default to the globally-configured value in the object.
+
 =item SSL => BOOL
 
 Optional. If true, an SSL connection will be used.
@@ -556,6 +566,17 @@ C<100-continue> and does not send the C<request_body> parameter until a
 C<100 Continue> response is received from the server. If an error response is
 received then the C<request_body> code, if present, will not be invoked.
 
+=item on_ready => CODE
+
+Optional. A callback that is invoked once a socket connection is established
+with the HTTP server, but before the request is actually sent over it. This
+may be used by the client code to inspect the socket, or perform any other
+operations on it. This code is expected to return a C<Future>; only once that
+has completed will the request cycle continue. If it fails, that failure is
+propagated to the caller.
+
+ $f = $on_ready->( $connection )
+
 =item on_redirect => CODE
 
 Optional. A callback that is invoked if a redirect response is received,
@@ -658,11 +679,16 @@ sub _do_one_request
    return $self->get_connection(
       host => $args{proxy_host} || $self->{proxy_host} || $host,
       port => $args{proxy_port} || $self->{proxy_port} || $port,
+      ( defined $args{family} ? ( family => $args{family} ) : () ),
       SSL  => $args{SSL},
       %{ $self->{ssl_params} },
       ( map { m/^SSL_/ ? ( $_ => $args{$_} ) : () } keys %args ),
    )->then( sub {
       my ( $conn ) = @_;
+      $args{on_ready} ? $args{on_ready}->( $conn )->then_done( $conn )
+                      : Future->done( $conn )
+   })->then( sub {
+      my ( $conn ) = @_;
 
       return $conn->request(
          request => $request,
